using System;
using System.Drawing;
using System.Runtime.InteropServices;

namespace RSSReaderPlugin.Controls
{

	/// <summary>
	/// Convert a standard Bitmap or Image to a transparent IPictureDisp.
	/// This class exists to fix an issue in SharpTools where icons that
	/// are assigned to buttons do not retain transparency information.
	/// This will be fully integrated in the next SharpTools release.
	/// </summary>
	public class IconHelper
	{

		#region Win32 API imports 

		[DllImport("user32.dll", EntryPoint="LoadBitmapA")]
		private static extern int LoadBitmap( int hInstance, int iBitmapIndex );

		[DllImport("oleaut32.dll", CharSet=CharSet.Unicode, PreserveSig=true, EntryPoint="OleLoadPictureFile")]
		private static extern void OleLoadPictureFile( object varFileName, [MarshalAs(UnmanagedType.IDispatch)] ref object lpIPictureDisp );

		[DllImport("oleaut32.dll", EntryPoint="OleCreatePictureIndirect")]
		private static extern int OleCreatePictureIndirect( ref PICTDESC pPictDesc, ref Guid riid, int fOwn, [MarshalAs(UnmanagedType.IDispatch)] ref object ppvObj );

		#region PICTDESC structure 

		private struct PICTDESC
		{

			public int		SizeOfStruct;
			public int		PicType;
			public IntPtr	hBitmap;
			public IntPtr	hPal;
			public int		Padding;

			public PICTDESC( IntPtr hBitmap )
			{

				this.PicType = 1;
				this.hBitmap = hBitmap;
				this.hPal = IntPtr.Zero;
				this.Padding = 0;

				// The two lines below look strange, but there is a reason:
				// Cannot use this.GetType() until all of the fields of 
				// a struct have been assigned to
				this.SizeOfStruct = 0;
				this.SizeOfStruct = Marshal.SizeOf( this.GetType() );

			}

		}

		#endregion 

		#endregion

		#region Constructor 

		private IconHelper()
		{
		}

		#endregion 

	}

}
